/*******************************************************************************
 * Licensed Materials - Property of IBM
 * FindImportantDates.java
 * (c) Copyright IBM Corporation 2012. 
 * U.S. Government Users Restricted Rights:  Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

package net.jazz.plainjava.find.important.dates;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.apache.commons.httpclient.URIException;
import org.apache.commons.httpclient.util.URIUtil;

import com.ibm.team.process.client.IProcessClientService;
import com.ibm.team.process.common.IDevelopmentLine;
import com.ibm.team.process.common.IDevelopmentLineHandle;
import com.ibm.team.process.common.IIteration;
import com.ibm.team.process.common.IIterationHandle;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProjectArea;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ILoginHandler2;
import com.ibm.team.repository.client.ILoginInfo2;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.client.login.UsernameAndPasswordLoginInfo;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * <p>
 * A project area is examined to find all of the development lines and iterations. The user provides
 * the name of the project area and the name of an output file where <a href="http://en.wikipedia.org/wiki/ICalendar">iCalendar</a> data
 * is generated. This file can be imported into a large variety of programs to show a complete calendar of
 * everything that is happening in your project. An optional parameter narrows it down to only the current iterations
 * to avoid too much calendar clutter.
 * </p>
 */
public class FindImportantDates {
	private static DateFormat ICS_DATE_FORMAT = new SimpleDateFormat("yyyyMMdd'T'HHmmss'Z'");
	static {
		ICS_DATE_FORMAT.setTimeZone(TimeZone.getTimeZone("GMT"));
	}
	private static String LINE_SUFFIX = "";
	static {
		if (System.getProperty("os.name").toUpperCase().indexOf("win") == -1) {
			LINE_SUFFIX = "\r";
		}
	}
	
	public static void main(final String[] args) throws TeamRepositoryException, URIException, URISyntaxException, FileNotFoundException {
		if (args.length < 5) {
			System.out.println("Please provide repository URI, username, password, project area name and iCalendar file as first five arguments to this program.");
			System.exit(1);
		}
		
		String repoUri = args[0];
		final String userId = args[1];
		final String password = args[2];
		
		TeamPlatform.startup();
		
		try {
			// Login to the repository using the provided credentials
			ITeamRepository repo = TeamPlatform.getTeamRepositoryService().getTeamRepository(repoUri);
			repo.registerLoginHandler(new ILoginHandler2() {
				@Override
				public ILoginInfo2 challenge(ITeamRepository arg0) {
					return new UsernameAndPasswordLoginInfo(userId, password);
				}
			});
			repo.login(new SysoutProgressMonitor());
			
			IProcessClientService proc = (IProcessClientService) repo.getClientLibrary(IProcessClientService.class);
			IItemManager itemManager = repo.itemManager();
			
			String projectAreaName = args[3];
			String encodedProjectAreaName = URIUtil.encodePath(projectAreaName);
			IProcessArea processArea = proc.findProcessArea(new URI(encodedProjectAreaName), IProcessClientService.ALL_PROPERTIES, new SysoutProgressMonitor());
			
			if (!(processArea instanceof IProjectArea)) {
				System.out.println("Provided project area name is not a project area. The name is being used for a team area on the server. Please try again with a project area name.");
				System.exit(1);
			}
			
			IProjectArea projectArea = (IProjectArea)processArea;
			
			File outputFile = new File(args[4]);
			PrintStream ps = new PrintStream(new FileOutputStream(outputFile));
			
			// Check to see if we are only generating "current" iterations
			boolean currentOnly = args.length > 5 && "current".equals(args[5]);
			
			ps.println("BEGIN:VCALENDAR"+LINE_SUFFIX);
			ps.println("VERSION:2.0"+LINE_SUFFIX);
			ps.println("PRODID:-//Jazz.net//NONSGML v1//EN"+LINE_SUFFIX);
			
			IDevelopmentLineHandle[] devLineHandles = ((IProjectArea)projectArea).getDevelopmentLines();
		    @SuppressWarnings("unchecked")
			List<IDevelopmentLine> devLines = itemManager.fetchCompleteItems(Arrays.asList(devLineHandles), IItemManager.DEFAULT, new SysoutProgressMonitor());

		    for (IDevelopmentLine devLine: devLines) {
		    	if (!currentOnly) {
		    		IIterationHandle[] iterationHandles = devLine.getIterations();
		    		
		    		writeDevLineData(ps, devLine, itemManager);
		    	
			    	@SuppressWarnings("unchecked")
					List<IIteration> iterations = itemManager.fetchCompleteItems(Arrays.asList(iterationHandles), IItemManager.DEFAULT, new SysoutProgressMonitor());
			    	
			    	for (IIteration iteration: iterations) {
			    		writeIterationData(ps, devLine.getLabel(), iteration, itemManager, true);
			    	}
		    	} else {
		    		IIterationHandle iterationHandle = devLine.getCurrentIteration();
		    		if (iterationHandle != null) {
			    		IIteration iteration = (IIteration) itemManager.fetchCompleteItem(iterationHandle, IItemManager.DEFAULT, new SysoutProgressMonitor());
			    		
			    		// Iterations can still be current for their development line but finished.
			    		if (iteration.getEndDate() == null || iteration.getEndDate().after(new Date())) {
			    			writeIterationData(ps, devLine.getLabel(), iteration, itemManager, false);
			    		}
		    		}
		    	}
		    }
		    
		    ps.println("END:VCALENDAR"+LINE_SUFFIX);
		    
		    System.out.println("Completed creating "+outputFile);
		} finally {
			TeamPlatform.shutdown();
		}
	}

	private static void writeIterationData(PrintStream ps, String suffix, IIteration iteration, IItemManager itemManager, boolean currentOnly) throws TeamRepositoryException {
		Date startDate = iteration.getStartDate();
		
		// With no start date there should be no event for this iteration.
		if (startDate == null) {
			System.out.println("WARNING: Iteration "+iteration.getLabel()+" has no start date. No event will be created.");
			return;
		}
		
		Date endDate = iteration.getEndDate();
		
		// It is an event if there is an end date
		// Otherwise, it is a todo to set an end date
		boolean makeEvent = (endDate != null) || !currentOnly;
		
		if (makeEvent) {
			ps.println("BEGIN:VEVENT"+LINE_SUFFIX);
		} else {
			ps.println("BEGIN:VTODO"+LINE_SUFFIX);
		}
		
		ps.println("UID:"+iteration.getItemId().getUuidValue()+LINE_SUFFIX);
		ps.println("DTSTAMP:"+ICS_DATE_FORMAT.format(startDate)+LINE_SUFFIX);
		ps.println("DTSTART:"+ICS_DATE_FORMAT.format(startDate)+LINE_SUFFIX);

		if (makeEvent) {
			ps.println("SUMMARY:"+(iteration.getLabel()+"/"+suffix)+LINE_SUFFIX);
			if (endDate != null) {
				ps.println("DTEND:"+ICS_DATE_FORMAT.format(endDate)+LINE_SUFFIX);
			} else {
				// If we are forced to make an event even though there is no end date
				//  then push off the end date to be a year from now at midnight.
				Calendar c = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
				c.setTime(new Date());
				c.add(Calendar.YEAR, 1);
				c.set(Calendar.AM_PM, Calendar.PM);
				c.set(Calendar.HOUR, 11);
				c.set(Calendar.MINUTE, 59);
				
				ps.println("DTEND:"+ICS_DATE_FORMAT.format(c.getTime())+LINE_SUFFIX);
				ps.println("DESCRIPTION:This event has no end date set."+LINE_SUFFIX);
			}
			ps.println("END:VEVENT"+LINE_SUFFIX);
		} else {
			System.out.println("WARNING: Iteration "+iteration.getLabel()+" has no end date set. A TODO has been created.");
			ps.println("SUMMARY:Set end date for "+(iteration.getLabel()+"/"+suffix)+LINE_SUFFIX);
			ps.println("PRIORITY:1"+LINE_SUFFIX);
			ps.println("STATUS:NEEDS-ACTION"+LINE_SUFFIX);
			ps.println("END:VTODO"+LINE_SUFFIX);
		}
		
		if (currentOnly) {
			// Recurse into the child iterations (if requested)
	    	@SuppressWarnings("unchecked")
			List<IIteration> childIterations = itemManager.fetchCompleteItems(Arrays.asList(iteration.getChildren()), IItemManager.DEFAULT, new SysoutProgressMonitor());
	
	    	for (IIteration childIteration: childIterations) {
	    		writeIterationData(ps, (iteration.getLabel()+"/"+suffix), childIteration, itemManager, true);
	    	}
		}
	}
	
	private static void writeDevLineData(PrintStream ps, IDevelopmentLine devLine, IItemManager itemManager) throws TeamRepositoryException {
		Date startDate = devLine.getStartDate();
		
		// With no start date there should be no event for this development line
		if (startDate == null) {
			System.out.println("WARNING: Iteration "+devLine.getLabel()+" has no start date. No event will be created.");
			return;
		}
		
		Date endDate = devLine.getEndDate();
		
		// It is an event if there is an end date
		// Otherwise, it is a todo to set an end date
		boolean makeEvent = (endDate != null);
		
		if (makeEvent) {
			ps.println("BEGIN:VEVENT"+LINE_SUFFIX);
		} else {
			ps.println("BEGIN:VTODO"+LINE_SUFFIX);
		}
		
		ps.println("UID:"+devLine.getItemId().getUuidValue()+LINE_SUFFIX);
		ps.println("DTSTAMP:"+ICS_DATE_FORMAT.format(startDate)+LINE_SUFFIX);
		ps.println("DTSTART:"+ICS_DATE_FORMAT.format(startDate)+LINE_SUFFIX);

		if (makeEvent) {
			ps.println("SUMMARY:"+devLine.getLabel()+LINE_SUFFIX);
			ps.println("DTEND:"+ICS_DATE_FORMAT.format(endDate)+LINE_SUFFIX);
			ps.println("END:VEVENT"+LINE_SUFFIX);
		} else {
			System.out.println("WARNING: Development line "+devLine.getLabel()+" has no end date. A TODO has been created for this.");
			ps.println("SUMMARY:Set end date for "+devLine.getLabel()+LINE_SUFFIX);
			ps.println("PRIORITY:1"+LINE_SUFFIX);
			ps.println("STATUS:NEEDS-ACTION"+LINE_SUFFIX);
			ps.println("END:VTODO"+LINE_SUFFIX);
		}
	}
}
